package ch.ethz.icr.culturemodel;

import uchicago.src.sim.gui.Drawable;
import uchicago.src.sim.gui.SimGraphics;

import java.awt.*;
import java.util.ArrayList;

/**
 * Axelrod's Culture Model
 *
 * Based on Axelrod, The Complexity of Cooperation, Chap. 7.
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @version 2.0
 */
public class Actor implements Drawable {
    // The agent's internal variables
    int x, y;                           // The player's location on the grid
    Model model;                        // Reference (handle) to the model
    int actorID;                       // The player's identification
    Actor other;                       // Handle to the opponent
    int culture[];                     // Culture vector
    ArrayList neighbors;               // list of von Neumann neighbors
    int region;                        // Regional index, used in region counter

    /**
     * Creating the agent
     *
     * @param id
     * @param m
     */
    public Actor(int id, Model m) {
        actorID = id;
        model = m;
        culture = new int[model.numFeatures];
        for (int i = 0; i < model.numFeatures; i++) {
            culture[i] = model.getNextIntFromTo(0, model.numTraits - 1);
        }
        neighbors = new ArrayList();
    }

    /**
     * Setting the agent's position on the grid
     *
     * @param a
     * @param b
     */
    public void placeTo(int a, int b) {
        x = a;
        y = b;
    }

    /**
     * Returns the cultural similarity to another actor a
     * @param a
     * @return No traits similar = 0.0;  all traits the same = 1.0
     */
    public double similarTo(Actor a) {
        int same = 0;
        for (int i = 0; i < model.numFeatures; i++)
            if (culture[i] == a.culture[i])
                same++;
        return (double) same / (double) model.numFeatures;
    }

    /**
     * Assimilation routine that follows Axelrod's algorithm
     * Note that the assimilation happens for a randomly chosen feature among
     * those features that are different.
     *
     * @param a
     */
    public void assimilate(Actor a) {
        int different[] = new int[model.numFeatures];
        int numDifferent = 0;
        for (int i = 0; i < model.numFeatures; i++) {
            if (culture[i] != a.culture[i]) {
                different[numDifferent] = i;
                numDifferent++;
            }
        }
        if (numDifferent > 0) {
            int feature = different[model.getNextIntFromTo(0, numDifferent - 1)];
            a.culture[feature] = culture[feature];
        }
    }

    public double culturalSimilarity(Actor a) {
        if (a != null) {
            double cultDist = similarTo(a);
            return cultDist;
        } else {
            return 1.0;
        }
    }

    /**
     * Returns the actor's culture vector as a string
     * @return
     */
    public String toString() {
        String result = "";
        for (int i = 0; i < model.numFeatures; i++) {
            result = result + culture[i];
        }
        return result;
    }

    public double culturalSimilarity() {
        double sum = 0;
        int count = 0;
        for (int i = 0; i < 4; i++) {
            Actor a = (Actor) getNeighbor(i);
            if (a != null) {
                double cultDist = similarTo(a);
                sum += cultDist;
                count++;
            }
        }
        return (sum / (double) count);
    }

    // This method returns the territorial neighbors based on grid access
    // pos = {0, 1, 2, 3}
    Actor getNeighbor(int pos) {
        // von Neumann coordinates used to find territorial neighbors
        final int offsets[][] = {{-1, 0}, {1, 0}, {0, -1}, {0, 1}};

        int xx = x + offsets[pos][0];
        int yy = y + offsets[pos][1];
        if ((xx >= 0) && (xx < model.world.getSizeX()) &&
                (yy >= 0) && (yy < model.world.getSizeY()))
            return (Actor) model.world.getObjectAt(xx, yy);
        else
            return null;
    }

    /////////////////////////////////////////////////////////////////////////
    // Functions of the GUI interface ///////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * Providing get methods for x in order to make them displayable
     * @return
     */
    public int getX() {
        return x;
    }

    /**
     * Providing get methods for y in order to make them displayable
     * @return
     */
    public int getY() {
        return y;
    }

    /**
     * Drawing the agent's 'dot' on the grid with the appropriate color
     * @param g
     */
    public void draw(SimGraphics g) {
        Color background = Color.white;
        g.drawRect(background);

        final Color left;
        final Color right;
        final Color top;
        final Color bottom;
        final Actor west = getNeighbor(0);
        final Actor east = getNeighbor(1);
        final Actor north = getNeighbor(2);
        final Actor south = getNeighbor(3);
        if (west != null) {
            int brightness = (int) ((culturalSimilarity(west)) * 255.0);
            left = new Color(brightness, brightness, brightness);
        } else {
            left = background;
        }
        if (east != null) {
            int brightness = (int) ((culturalSimilarity(east)) * 255.0);
            right = new Color(brightness, brightness, brightness);
        } else {
            right = background;
        }
        if (north != null) {
            int brightness = (int) ((culturalSimilarity(north)) * 255.0);
            top = new Color(brightness, brightness, brightness);
        } else {
            top = background;
        }
        if (south != null) {
            int brightness = (int) ((culturalSimilarity(south)) * 255.0);
            bottom = new Color(brightness, brightness, brightness);
        } else {
            bottom = background;
        }
        g.draw4ColorHollowRect(top, bottom, left, right);
    }

}