package ch.ethz.icr.culturemodel;

import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.engine.SimpleModel;
import uchicago.src.sim.space.Object2DGrid;

import java.awt.*;

/**
 * Axelrod's Culture Model
 *
 * Based on Axelrod, The Complexity of Cooperation, Chap. 7.
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @version 2.0
 */
public class Model extends SimpleModel {
    // Model variables
    Object2DGrid world;                  // The 2D grid representing the world

    // Main parameters
    int worldSize;                       // The size of the world's 'sides'
    int numFeatures;                     // The number of sites in culture vector
    int numTraits;                       // The number of possible values per feature

    // Other variables
    int numActors;                       // The number of agents (based on worldSize)
    int actorID;                         // Variable to store the ID of the
    Color[] color;                       // Color lookup table

    /**
     * Initializing the model
     */
    public void setup() {
        // The setup() method of the parent MUST be called first!
        super.setup();

        // Setting the name of our model
        name = "CultureModel";

        // Setting the values of the model parameters
        worldSize = 10;
        numFeatures = 5;
        numTraits = 10;
    }

    /**
     * The method to build the Model's internals
     */
    public void buildModel() {
        // Creating the world
        world = new Object2DGrid(worldSize, worldSize);
        // Calculating the number of players (one player per grid cell)
        numActors = worldSize * worldSize;

        int actorID = 0;   // The ID of the agent under creation

        // Creating the agents and placing them in the grid.
        for (int x = 0; x < worldSize; x++)
            for (int y = 0; y < worldSize; y++) {
                actorID++;
                Actor anActor = new Actor(actorID, this);
                agentList.add(anActor);
                world.putObjectAt(x, y, anActor); // This call places the agent
                anActor.placeTo(x, y); // The player has to record its own position
            }

        // Create the neighborhood lists
        for (int x = 0; x < worldSize; x++) {
            for (int y = 0; y < worldSize; y++) {
                Actor anActor = (Actor) world.getObjectAt(x, y);
                anActor.neighbors.addAll(world.getVonNeumannNeighbors(x, y, false));
            }
        }
        reportResults();
    }

    /////////////////////////////////////////////////////////////////////////
    //  Iterated methods ////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * The activities of the timestep
     * Main step routine with sequential activation
     * An actor1 is picked randomly and an actor2 is stochastically selected from neighborhood
     * Axelrod's key rule stipulates that the assimilation happens with the same probability as
     * the similarity rate
     */
    public void step() {
        Actor actor1 = (Actor) agentList.get(getNextIntFromTo(0, numActors - 1));
        Actor actor2 = (Actor) actor1.neighbors.get(getNextIntFromTo(0, actor1.neighbors.size() - 1));
        if (getNextDoubleFromTo(0.0, 1.0) < actor1.similarTo(actor2))
            actor1.assimilate(actor2);

        if (getTickCount() % 100 == 0)
            reportResults();
    }

    /**
     * Routine that shows the culture map
     */
    public void reportResults() {
        System.out.println("Time = " + getTickCount());
        for (int x = 0; x < worldSize; x++) {
            for (int y = 0; y < worldSize; y++) {
                Actor a = (Actor) world.getObjectAt(x, y);
                System.out.print(a.toString() + " ");
            }
            System.out.println();
        }
    }

    /**
     * Creating and starting your model
     * @param args
     */
    public static void main(String[] args) {
        SimInit init = new SimInit();
        Model m = new Model();
        init.loadModel(m, null, false);
    }
}